from django.conf import settings

from .._utils import *
from ...images.sorl import get_preset_args


class PresetTestCase(ArmContentTestCase):

    def setUp(self):
        if hasattr(settings, 'ARMSTRONG_PRESETS'):
            self._previous_presets = settings.ARMSTRONG_PRESETS
        else:
            self._previous_presets = None

        settings.ARMSTRONG_PRESETS = {
            'width_only': {'width': 200},
            'height_only': {'height': 300},
            'width_and_height': {
                'width': 75,
                'height': 100,
            },
        }

    def tearDown(self):
        if self._previous_presets is not None:
            settings.ARMSTRONG_PRESETS = self._previous_presets

    def test_width_only(self):
        args = get_preset_args('width_only')
        self.assertEqual(args['dimensions'], '200')

    def test_height_only(self):
        args = get_preset_args('height_only')
        self.assertEqual(args['dimensions'], 'x300')

    def test_width_and_height(self):
        args = get_preset_args('width_and_height')
        self.assertEqual(args['dimensions'], '75x100')



from copy import deepcopy

from django.conf import settings
import fudge
import fudge.patcher
from lettuce import *
from sorl.thumbnail.base import ThumbnailBackend

from armstrong.core.arm_content.images.sorl import get_preset_thumbnail, dimensions
from armstrong.core.arm_content.tests.arm_content_support.models import SorlImage


def get_thumbnail_mock(self, file_, dimensions, **kwargs):
    thumbnail = deepcopy(file_)
    thumbnail.storage = file_.storage  # This doesn't copy over for some reason.
    thumbnail._kwargs = kwargs
    thumbnail._kwargs['dimensions'] = dimensions
    return thumbnail

@before.each_scenario
def initialize_world(scenario):
    world.presets = None
    world.defaults = None

@step(u'Given I have an Image that refers to (.*)')
def given_i_have_an_image_that_refers_to_path(step, path):
    world.image = SorlImage(image=path)

@step(u'And I have the following thumbnail presets:')
def and_i_have_the_following_thumbnail_presets(step):
    world.presets = {}
    for hash in step.hashes:
        name = hash.pop('name')
        # Empty columns should be treated as completely omitted.
        for key, value in hash.items():
            if not value:
                del hash[key]
        world.presets[name] = hash

@step(u'When I ask for each preset thumbnail for the image')
@fudge.patcher.with_patched_object(ThumbnailBackend, 'get_thumbnail', get_thumbnail_mock)
def when_i_ask_for_each_preset_thumbnail_for_the_image(step):
    world.thumbnails = dict(
        (preset, get_preset_thumbnail(world.image.image, preset,
            presets=world.presets, defaults=world.defaults))
        for preset in world.presets)

@step(u'When I ask for the (.*) thumbnail for the image')
@fudge.patcher.with_patched_object(ThumbnailBackend, 'get_thumbnail', get_thumbnail_mock)
def when_i_ask_for_a_preset_thumbnail_for_the_image(step, preset_label):
    world.thumbnail = get_preset_thumbnail(world.image.image, preset_label,
        presets=world.presets, defaults=world.defaults)

@step(u'Then the returned thumbnail is the original image')
def then_the_returned_thumbnail_is_the_original_image(step):
    assert world.thumbnail.name == world.image.image.name

@step(u'Then each thumbnail has the specified settings')
def then_each_thumbnail_has_the_specified_settings(step):
    for preset_label, preset in world.presets.items():
        preset = preset.copy()
        thumbnail = world.thumbnails[preset_label]

        # Use the arguments passed to sorl's get_thumbnail as an approximation of
        # getting a thumbnail returned with those settings.
        args = thumbnail._kwargs
        assert args['dimensions'] == dimensions(preset.get('width'), preset.get('height'))
        preset.pop('height', None), preset.pop('width', None)
        for key, value in preset.items():
            assert args[key] == value

@step(u'And I have a default preset quality of 100')
def and_i_have_a_default_preset_quality_of_100(step):
    world.defaults = world.defaults or {}
    world.defaults['quality'] = 100

@step(u'And each thumbnail has a quality of 100')
def and_each_thumbnail_has_a_quality_of_100(step):
    for thumbnail in world.thumbnails.values():
        assert thumbnail._kwargs['quality'] == 100

@step(u'And the thumbnails without specified quality settings have a quality of 100')
def and_the_thumbnails_without_specified_quality_settings_have_a_quality_of_100(step):
    for preset_label, preset in world.presets.items():
        thumbnail = world.thumbnails[preset_label]
        if 'quality' not in preset:
            assert thumbnail._kwargs['quality'] == 100

@step(u'When I render its (\w+) thumbnail')
@fudge.patcher.with_patched_object(ThumbnailBackend, 'get_thumbnail', get_thumbnail_mock)
def when_i_render_its_thumbnail(step, preset_label):
    world.rendered = world.image.render_visual(preset_label, presets=world.presets,
        defaults=world.defaults)

@step(u'Then I see an IMG tag')
def then_i_see_an_img_tag(step):
    assert world.rendered.startswith('<img')



from django.utils.safestring import mark_safe
from django.forms.widgets import ClearableFileInput
from django.conf import settings


STATIC_URL = getattr(settings, settings.STATIC_URL, "/static/")


class AudioFileWidget(ClearableFileInput):
    class Media:
        css = {
                "all": [STATIC_URL + "skin/jplayer.blue.monday.css",]
        }
        js = [STATIC_URL + "js/jquery.jplayer.min.js",]
 
    def render(self, name, value, attrs):
        self.attrs=attrs
        parent_output = super(AudioFileWidget, self).render(name, value, attrs)
        from  ...fields.audio import AudioFile
        if type(value) is AudioFile:
            template_player = value.render()
        else:
            template_player = ''

        return mark_safe(parent_output + (template_player))



from django.conf import settings
from django.db import models
from ..fields import AudioField


UPLOAD_TO = getattr(settings, "MEDIA_ROOT", "./media")

class AudioMixin(models.Model):
    file = AudioField(upload_to='%saudio/' % UPLOAD_TO)
    playtime = models.PositiveIntegerField("playtime in seconds",
                                           null=True,
                                           blank=True)

    filetype = models.CharField("filetype",
                                max_length=16,
                                null=True,
                                blank=True)

    artist = models.CharField("artist",
                              max_length=100,
                              null=True,
                              blank=True,
                             )

    genre = models.CharField("genre",
                              max_length=100,
                              null=True,
                              blank=True,
                            )

    def __unicode__(self):
        return unicode(self.__str__())

    def __str__(self):
        return "%s by %s on %s" % (self.file, self.artist, self.album)

    @property
    def html(self):
        return self.file.render()

    class Meta:
        abstract = True



from ..._utils import *

from ....fields.video import EmbeddedVideo
from ....video.backends import helpers
from ....video.backends.youtube import YouTubeBackend


class YouTubeBackendTestCase(ArmContentTestCase):
    def generate_random_url(self):
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        return random_id, url

    def test_returns_tuple_with_url_as_first_value(self):
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        self.assertEqual("http", video.url.scheme)
        self.assertEqual("youtube.com", video.url.netloc)

    def test_returns_tuple_with_id_as_second_value(self):
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        self.assertEqual(random_id, video.id)

    def test_returns_the_expected_html_when_embed_is_called(self):
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        expected = "".join([
            '<iframe title="YouTube video player" ',
            'width="640" height="390" ',
            'src="http://www.youtube.com/embed/%s" ',
            'frameborder="0" allowfullscreen></iframe>']) % random_id
        self.assertEqual(expected, backend.embed(video))

    def test_embed_width_can_be_set_with_a_kwarg(self):
        random_width = random.randint(1000, 2000)
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        expected = "".join([
            '<iframe title="YouTube video player" ',
            'width="%d" height="390" ' % random_width,
            'src="http://www.youtube.com/embed/%s" ',
            'frameborder="0" allowfullscreen></iframe>']) % random_id
        self.assertRegexpMatches(backend.embed(video, width=random_width),
                r'width="%d"' % random_width)

    def test_embed_height_can_be_set_with_a_kwarg(self):
        random_height = random.randint(1000, 2000)
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        expected = "".join([
            '<iframe title="YouTube video player" ',
            'width="%d" height="390" ' % random_height,
            'src="http://www.youtube.com/embed/%s" ',
            'frameborder="0" allowfullscreen></iframe>']) % random_id
        self.assertRegexpMatches(backend.embed(video, height=random_height),
                r'height="%d"' % random_height)

    def test_embed_width_and_height_can_be_strings(self):
        random_height = str(random.randint(1000, 2000))
        random_width = str(random.randint(1000, 2000))
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = YouTubeBackend()

        video = EmbeddedVideo(url, backend)
        expected = "".join([
            '<iframe title="YouTube video player" ',
            'width="%s" height="%s" ' % (random_width, random_height),
            'src="http://www.youtube.com/embed/%s" ',
            'frameborder="0" allowfullscreen></iframe>']) % random_id
        self.assertRegexpMatches(backend.embed(video, width=random_width),
                r'width="%s"' % random_width)
        self.assertRegexpMatches(backend.embed(video, height=random_height),
                r'height="%s"' % random_height)

    def test_height_defaults_to_configured_if_not_provided(self):
        random_height = random.randint(1000, 2000)
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_HEIGHT=random_height)
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH="does not matter")

        with fudge.patched_context(helpers, 'settings', settings):
            random_id, url = self.generate_random_url()
            backend = YouTubeBackend()
            video = EmbeddedVideo(url, backend)
            self.assertRegexpMatches(backend.embed(video),
                    r'height="%s"' % random_height)

    def test_width_defaults_to_configured_if_not_provided(self):
        random_width = random.randint(1000, 2000)
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH=random_width)
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_HEIGHT="does not matter")

        with fudge.patched_context(helpers, 'settings', settings):
            random_id, url = self.generate_random_url()
            backend = YouTubeBackend()
            video = EmbeddedVideo(url, backend)
            self.assertRegexpMatches(backend.embed(video),
                    r'width="%s"' % random_width)



from datetime import datetime
from django.core.files import File
from django.conf import settings

from armstrong.dev.tests.utils import ArmstrongTestCase
from armstrong.dev.tests.utils.concrete import *
from armstrong.dev.tests.utils.users import *

import fudge

from .arm_content_support.models import AudioModel

class ArmContentTestCase(ArmstrongTestCase):
    pass


def add_n_users_to_authors(authors, *users):
    for user in users:
        authors.users.add(user)


def add_profile_to(profile_class, *users):
    for user in users:
        profile = profile_class.objects.create(user=user)
        user._profile_cache = profile


def add_authors_to(model, *authors):
    for author in authors:
        model.authors.add(author)


def random_authored_model(klass, *authors):
    article = klass.objects.create()
    add_authors_to(article, *authors)
    return article


def load_audio_model(filename, file_field_name='file', model=AudioModel, model_args={}):
    f = open(settings.STATIC_ROOT + 'audio/' + filename, "rb+")
    uf = File(file=f)
    model_args[file_field_name] = uf
    am = model(**model_args)
    am.save()
    return am





from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models.fields.related import create_many_related_manager
from django.db.models.fields.related import ManyRelatedObjectsDescriptor


# TODO: find permanent home for this code
def user_to_link(user):
    try:
        return '<a href="%s">%s</a>' % (user.get_profile().get_absolute_url(),
                user.get_full_name())
    except ObjectDoesNotExist:
        return user_to_name(user)


# TODO: find permanent home for this code
def user_to_name(user):
    return user.get_full_name()


class AuthorsManager(User.objects.__class__):
    def has_usable_override(self):
        return hasattr(self.instance, self.override_field_name) and \
                len(getattr(self.instance, self.override_field_name)) > 0

    def has_usable_extra(self):
        return hasattr(self.instance, self.extra_field_name)

    def __unicode__(self, formatter=user_to_name):
        if self.has_usable_override():
            return getattr(self.instance, self.override_field_name)

        names = [formatter(a) for a in self.all()]
        extra = False
        if self.has_usable_extra():
            extra = getattr(self.instance, self.extra_field_name)

        # Warning: weird logic ahead.  It's assumed that ``extra`` has the
        # final conjunction, so we have to join everything by commas.  If
        # there's no ``extra``, we must join the final elements by ``" and "``.
        #
        # TODO: Make this use a template for rendering so its easier to
        # international later on.
        ret = u', '.join(names[:-2] + \
                [(u', ' if extra else u' and ').join(names[-2:])])

        # This adds a space if ``extra`` starts with a letter.  This assumes
        # that anything other than a letter (generally, a comma) should be put
        # directly against the byline.
        if extra:
            space = ' ' if extra[0].isalpha() else ''
            ret = u"%s%s%s" % (ret, space, extra)
        return ret

    def __str__(self):
        return unicode(self)

    def html(self):
        return self.__unicode__(formatter=user_to_link)


class AuthorsDescriptor(object):
    def __init__(self, m2m_field):
        self.field = m2m_field

    def __get__(self, instance, instance_type=None):
        if not instance:
            return self

        RelatedManager = create_many_related_manager(AuthorsManager,
                self.field.rel)
        core_filters = {
            '%s__pk' % self.field.related_query_name(): instance._get_pk_val(),
        }
        manager = RelatedManager(
            model=self.field.rel.to,
            core_filters=core_filters,
            instance=instance,
            symmetrical=self.field.rel.symmetrical,
            source_field_name=self.field.m2m_field_name(),
            target_field_name=self.field.m2m_reverse_field_name(),
            reverse=False,
        )

        # Set this after the fact because AuthorsManager is the superclass and
        # RelatedManager doesn't know about these attributes
        manager.override_field_name = self.field.override_field_name
        manager.extra_field_name = self.field.extra_field_name

        return manager

    def __set__(self, instance, value):
        if instance is None:
            raise AttributeError("Manager must be accessed via instance")

        if not self.field.rel.through._meta.auto_created:
            opts = self.field.rel.through._meta
            raise AttributeError("Cannot set values on a ManyToManyField which specifies an intermediary model. Use %s.%s's Manager instead." % (opts.app_label, opts.object_name))

        manager = self.__get__(instance)
        manager.clear()
        manager.add(*value)


class AuthorsField(models.ManyToManyField):
    def __init__(self, to=None, override_field_name='authors_override',
            extra_field_name='authors_extra', **kwargs):
        if not to:
            to = User
        self.override_field_name = override_field_name
        self.extra_field_name = extra_field_name
        super(AuthorsField, self).__init__(to=to, **kwargs)

    def contribute_to_class(self, cls, name):
        super(AuthorsField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, AuthorsDescriptor(self))

    def south_field_triple(self):
        from south.modelsinspector import introspector
        field_class = "%s.%s" % (self.__class__.__module__,
                self.__class__.__name__)
        args, kwargs = introspector(self)
        kwargs.update({
            'override_field_name': "'%s'" % self.override_field_name,
            'extra_field_name': "'%s'" % self.extra_field_name,
        })
        return (field_class, args, kwargs)



# coding=utf-8
from django.conf import settings

from ..arm_content_support.models import OverrideAudioModel
from ..arm_content_support.forms import AudioModelForm

from .._utils import *
from ...fields.widgets.audio import AudioFileWidget

__all__ = ["Mp3Test"]

class AudioFieldMetadataTestCase(ArmContentTestCase):
    def setUp(self):
        self.audio_model = load_audio_model(self.filename)
        self.override_audio_model = load_audio_model(self.filename,
                model=OverrideAudioModel)

    def test_audiofield_default_widget(self):
        """
        test that the correct widget is displayed in
        a model form
        """
        form = AudioModelForm()
        self.assertTrue(
            type(form.fields['file'].widget) is
            AudioFileWidget)
        form2 = AudioModelForm(initial={
                               "file": self.audio_model.file})
        self.assertTrue(self.audio_model.file.url in form2.as_ul())
        self.assertTrue("jplayer" in form2.as_ul())
                

    def test_audiofield_filetype(self):
        """
        test that the file type returned by the file field is correct
        """
        self.assertEqual(self.audio_model.file.filetype, self.filetype)

    def test_audiofield_metadata(self):
        """
        confirm that the extracted metadata matches the expected values
        """
        for key in self.audio_metadata.keys():
            self.assertEqual(
                    self.audio_metadata[key],
                    self.audio_model.file.metadata[key])

    def test_audio_model_overriding(self):
            '''test the audio model field pre population'''
            self.override_audio_model.save()
            unoverridden = OverrideAudioModel.objects.get(
                                        pk=self.override_audio_model.pk)

            self.assertEqual(self.audio_metadata['artist'], unoverridden.artist)
            self.override_audio_model.artist = 'qr'
            self.override_audio_model.save()

            overridden = OverrideAudioModel.objects.get(pk=self.override_audio_model.pk)

            self.assertEqual('qr', overridden.artist)


class Mp3Test(AudioFieldMetadataTestCase):
    filename = 'test.mp3'
    filetype = 'mp3'
    playtime = '4'
    audio_metadata = {'album': u'from the internet',
                     'title': u'im chargin my laser',
                     'artist': u'armstrong',
                     'genre': u'Satire',
                     'date': u'2011',
                     'tracknumber': u'1'}




from django.contrib import admin
from .models import *


admin.site.register(SimpleAuthoredModel)
admin.site.register(AudioModel)
admin.site.register(OverrideAudioModel)
admin.site.register(SorlImage)



from django.db import models
from ..fields import AuthorsField


class AuthorsMixin(models.Model):
    authors = AuthorsField(blank=True)
    authors_extra = models.CharField(blank=True, max_length=200)
    authors_override = models.CharField(blank=True, max_length=200)

    class Meta:
        abstract = True



from django.db.models import signals
from django.db.models.fields.files import FileField, FieldFile, FileDescriptor
from django.forms.fields import FileField as FileFormField
from django.template.loader import get_template
from django.template import Context

from armstrong.core.arm_content.fields.widgets import AudioFileWidget
from armstrong.utils.backends import GenericBackend


class AudioFile(FieldFile):
    """
    A mixin for use alongside django.core.files.base.File, which provides
    additional features for dealing with audio files.
    """

    def __init__(self, *args, **kwargs):
        super(AudioFile, self).__init__(*args, **kwargs)
        #allows the override of anything that follows the _attrname caching pattern
        for attr in dir(self):
            if attr in kwargs:
                setattr(self, '_' + attr, kwargs[attr])
        self.backend = \
            GenericBackend('ARMSTRONG_EXTERNAL_AUDIO_METADATA_BACKEND')\
            .get_backend()

    def _transcode(self, toformat):
        """
        returns a transcoded file
        perhaps should do more complex and different things
        like transcode via zencoder
        """
        raise NotImplementedError

    def render(self, *args, **kwargs):
        audio_player_template = get_template('audio/player.html')
        return audio_player_template.render(
                            Context(
                                {'url': self.url,
                                'filetype': self.filetype,
                                'name': self.metadata['title'],
                                'fileno': str(self.fileno())
                                }
                            )
                        )

    @property
    def filetype(self):
        """
        get the encoding of the file
        """
        if not hasattr(self, '_filetype'):
            self._filetype = self.backend.filetype(self.file)
        return self._filetype

    @property
    def playtime(self):
        """
        get the playtime of the file
        """
        if not hasattr(self, '_playtime'):
            self._playtime = self.backend.playtime(self.file)
        return self._playtime

    @property
    def bitrate(self):
        """
        get the bit rate
        """
        if not hasattr(self, '_bitrate'):
            self._bitrate = self.backend.bitrate(self.file)
        return self._bitrate

    @property
    def metadata(self):
        """
        get the all metadata as a dictionary
        """
        if not hasattr(self, '_metadata'):
            self._metadata = self.backend.metadata(self.file)
        return self._metadata


class AudioFileDescriptor(FileDescriptor):
    """
    Just like the FileDescriptor, but for AudioFields. The only difference is
    assigning the audio metadata to the metadata dict, if appropriate.
    """
    def __set__(self, instance, value):
        previous_file = instance.__dict__.get(self.field.name)
        super(AudioFileDescriptor, self).__set__(instance, value)
        if previous_file is not None:
            self.field.update_metadata(instance, )


class AudioFormField(FileFormField):
    widget = AudioFileWidget


class AudioField(FileField):
    attr_class = AudioFile
    descriptor_class = AudioFileDescriptor

    def __init__(self, *args, **kwargs):
        self.overrides = {}
        for key in kwargs.keys():
            if key.endswith('_field_name'):
                self.overrides[key] = kwargs[key]
                del(kwargs[key])
        return super(AudioField, self).__init__(self, *args, **kwargs)

    def __unicode__(self):
        return self.__str__()

    def __str__(self):
        return self.name

    def contribute_to_class(self, cls, name):
        super(AudioField, self).contribute_to_class(cls, name)
        # Attach update_metadata so that dimension fields declared
        # after their corresponding image field don't stay cleared by
        # Model.__init__, see bug #11196.
        signals.post_init.connect(self.update_metadata, sender=cls)

    def formfield(self, **kwargs):
        # This is a fairly standard way to set up some defaults
        # while letting the caller override them.
        defaults = {'form_class': AudioFormField}
        defaults.update(kwargs)
        return super(FileField, self).formfield(**defaults)

    def update_metadata(self, instance, *args, **kwargs):
        """
        update the metadata  IF a file has been set
        instance == the model

        """
        audio_file = getattr(instance, self.name)
        if hasattr(audio_file, "file"):
            for key in audio_file.metadata.keys():
                field_name = self.overrides.get(key + '_field_name', key)
                if not (hasattr(instance, field_name) and  getattr(instance, field_name)):
                    setattr(instance, field_name, audio_file.metadata[key])



from django.forms import ModelForm
from .models import SimpleMixedinAuthorModel
from .models import AudioModel


class AudioModelForm(ModelForm):
    class Meta:
        model=AudioModel


class SimpleMixedinAuthorForm(ModelForm):
    class Meta:
        model = SimpleMixedinAuthorModel



# coding=utf-8
from django.db import models
from .._utils import *

from ..arm_content_support.models import ConcreteArticle

import datetime

class PublicationManagerTestCase(ArmContentTestCase):
    def test_published_manager_only_pulls_published_content(self):
        published = ConcreteArticle.objects.create(title="Published",
                                           pub_date=datetime.datetime.now()-datetime.timedelta(days=1),
                                           pub_status='P')
        draft_art = ConcreteArticle.objects.create(title="Not Published",
                                           pub_date=datetime.datetime.now()-datetime.timedelta(days=1),
                                           pub_status='D')
        scheduled = ConcreteArticle.objects.create(title="Future Published",
                                           pub_date=datetime.datetime.now()+datetime.timedelta(days=1),
                                           pub_status='P')
        all_published = ConcreteArticle.published.all().select_subclasses()
        self.assertTrue(published in all_published)
        self.assertTrue(not draft_art in all_published)
        self.assertTrue(not scheduled in all_published)



from django.conf import settings
from django.conf.urls.defaults import patterns, include, url
from django.conf.urls.static import static
from django.contrib import admin


admin.autodiscover()


urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
) + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import pkg_resources
pkg_resources.declare_namespace(__name__)

from .authors import AuthorsMixin
from .audio import AudioMixin
from .images import *
from .publication import PublicationMixin
from .video import EmbeddedVideoMixin



AUTHORS = ('Author Information', {
    'fields': ('authors', 'authors_override', 'authors_extra'),
})

PUBLICATION = ('Publication Information', {
    'fields': ('pub_date', 'pub_status', 'sites', ),
})

TAXONOMY = ('Taxonomy', {
    'fields': ('sections', 'tags', ),
})



# coding=utf-8
from django.db import models
from .._utils import *

from ..arm_content_support.models import AudioMixinModel

from ...fields import AudioField
from ..fields.audio import Mp3Test

AUDIO_FIELDS = {'file':AudioField,
                'playtime':models.PositiveIntegerField,
                'filetype':models.CharField, 
                'artist':models.CharField,
                'genre':models.CharField}

class AudioMixinTestCase(ArmContentTestCase):
    def test_models_mixed_in_with_AudioMixin_has_audio_fields(self):
        model = AudioMixinModel.objects.create()
        for (field, field_type) in AUDIO_FIELDS.items():
            self.assertModelHasField(model, field, field_type)

    def test_audio_model_works_when_cast_to_string(self):
        am = load_audio_model(filename = 'test.mp3', model = AudioMixinModel)        
        expected = "%s by %s on %s" % (am.file, am.artist, am.album)
        self.assertEqual(str(am), expected)

    def test_should_be_able_to_convert_to_unicode(self):
        am = load_audio_model(filename = 'test.mp3', model = AudioMixinModel)        
        expected = u"%s by %s on %s" % (am.file, am.artist, am.album)

        self.assertTrue(type(am.__unicode__()) is unicode)
        self.assertEqual(am.__unicode__(), expected)

    def test_html_player_output(self):
        am = load_audio_model(filename = 'test.mp3', model = AudioMixinModel)        
        self.assertEqual(am.html, am.file.render() )



from ..._utils import *

from ....fields.video import EmbeddedVideo
from ....video.backends import helpers
from ....video.backends import vimeo
from ....video.backends.vimeo import VimeoBackend


class VimeoBackendTestCase(ArmContentTestCase):
    def generate_random_url(self):
        random_id = random.randint(1000, 2000)
        url = "http://vimeo.com/%d" % random.randint(1000, 2000)
        return (random_id, url)

    def test_prepare_sets_url_on_provided_video(self):
        backend = VimeoBackend()
        url = "http://vimeo.com/%d" % random.randint(1000, 2000)

        video = EmbeddedVideo(url, backend)
        self.assertEqual(video.url.scheme, "http")
        self.assertEqual(video.url.netloc, "vimeo.com")

    def test_prepare_sets_id_on_provided_video(self):
        random_id = str(random.randint(1000, 2000))
        backend = VimeoBackend()
        url = "http://vimeo.com/%s" % random_id

        video = EmbeddedVideo(url, backend)
        self.assertEqual(video.id, random_id)

    def test_returns_none_if_cannot_parse(self):
        unparseable_url = "foo.bar.foo:baz"
        backend = VimeoBackend()
        video = EmbeddedVideo(backend=backend)
        video.raw_url = unparseable_url

        self.assertNone(backend.prepare(video))

    def test_returns_true_if_can_parse(self):
        random_id = str(random.randint(1000, 2000))
        backend = VimeoBackend()
        url = "http://vimeo.com/%s" % random_id
        video = EmbeddedVideo(backend=backend)
        video.raw_url = url
        self.assertTrue(backend.prepare(video))

    def test_returns_none_if_video_does_not_have_raw_url(self):
        backend = VimeoBackend()
        video = EmbeddedVideo(backend=backend)
        self.assertNone(backend.prepare(video))

    def test_video_is_not_modified_if_backend_fails(self):
        backend = VimeoBackend()
        unparseable_url = "foo.bar.foo:baz"
        video = EmbeddedVideo(backend=backend)
        video.raw_url = unparseable_url

        self.assertDoesNotHave(video, "url", msg="sanity check")
        backend.prepare(video)
        self.assertDoesNotHave(video, "url")

    def test_embed_returns_expected_html_when_called(self):
        random_id = str(random.randint(1000, 2000))
        url = "http://vimeo.com/%s" % random_id
        expected_url = ''.join([
            'http://player.vimeo.com/video/%s' % random_id,
            '?title=0&amp;byline=0&amp;portrait=0'])
        expected = "".join([
            '<iframe src="%s" ' % expected_url,
            'width="640" height="390" frameborder="0"></iframe>'
        ])

        backend = VimeoBackend()
        video = EmbeddedVideo(url, backend=backend)

        self.assertEqual(backend.embed(video), expected)

    def test_embed_can_change_height_with_kwarg(self):
        random_id = str(random.randint(1000, 2000))
        random_height = random.randint(1000, 2000)
        url = "http://vimeo.com/%s" % random_id
        expected_url = ''.join([
            'http://player.vimeo.com/video/%s' % random_id,
            '?title=0&amp;byline=0&amp;portrait=0'])
        expected = "".join([
            '<iframe src="%s" ' % expected_url,
            'width="398" height="%s" frameborder="0"></iframe>' \
                    % random_height,
        ])

        backend = VimeoBackend()
        video = EmbeddedVideo(url, backend=backend)

        self.assertRegexpMatches(backend.embed(video, height=random_height),
                r'height="%d"' % random_height)

    def test_embed_can_change_width_with_kwarg(self):
        random_id = str(random.randint(1000, 2000))
        random_width = random.randint(1000, 2000)
        url = "http://vimeo.com/%s" % random_id
        expected_url = ''.join([
            'http://player.vimeo.com/video/%s' % random_id,
            '?title=0&amp;byline=0&amp;portrait=0'])
        expected = "".join([
            '<iframe src="%s" ' % expected_url,
            'width="%d" height="224" frameborder="0"></iframe>' % random_width,
        ])

        backend = VimeoBackend()
        video = EmbeddedVideo(url, backend=backend)

        self.assertRegexpMatches(backend.embed(video, width=random_width),
                r'width="%d"' % random_width)

    def test_embed_width_and_height_can_be_strings(self):
        random_height = str(random.randint(1000, 2000))
        random_width = str(random.randint(1000, 2000))
        random_id = str(random.randint(100, 200))
        url = "http://youtube.com/watch?v=%s" % random_id
        backend = VimeoBackend()

        url = "http://vimeo.com/%s" % random_id
        video = EmbeddedVideo(url, backend)

        self.assertRegexpMatches(backend.embed(video, width=random_width),
                r'width="%s"' % random_width)
        self.assertRegexpMatches(backend.embed(video, height=random_height),
                r'height="%s"' % random_height)

    def test_height_defaults_to_configured_if_not_provided(self):
        random_height = random.randint(1000, 2000)
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_HEIGHT=random_height)
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH="does not matter")

        with fudge.patched_context(helpers, 'settings', settings):
            random_id, url = self.generate_random_url()
            backend = VimeoBackend()
            video = EmbeddedVideo(url, backend)
            self.assertRegexpMatches(backend.embed(video),
                    r'height="%s"' % random_height)

    def test_width_defaults_to_configured_if_not_provided(self):
        random_width = random.randint(1000, 2000)
        settings = fudge.Fake()
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_WIDTH=random_width)
        settings.has_attr(ARMSTRONG_EMBED_VIDEO_HEIGHT="does not matter")

        with fudge.patched_context(helpers, 'settings', settings):
            random_id, url = self.generate_random_url()
            backend = VimeoBackend()
            video = EmbeddedVideo(url, backend)
            self.assertRegexpMatches(backend.embed(video),
                    r'width="%s"' % random_width)



from __future__ import absolute_import

from sorl.thumbnail import get_thumbnail

from . import presets as presets_utils


def dimensions(width, height):
    s = '%sx%s' % (width if width is not None else '',
        height if height is not None else '')
    # If the width was omitted, we have 'x%(height)s', which is what sorl
    # expects. If the height was omitted, we have '%(width)sx'. sorl
    # expects just width, so strip trailing 'x' characters.
    return s.rstrip('x')

def get_preset_args(preset_label, presets=None, defaults=None):
    args = presets_utils.get_preset_args(preset_label, presets, defaults)
    width = args.pop('width') if 'width' in args else ''
    height = args.pop('height') if 'height' in args else ''
    args['dimensions'] = dimensions(width, height)
    return args

def get_preset_thumbnail(file_, preset_label, presets=None, defaults=None):
    '''A thin wrapper around sorl's get_thumbnail that allows for presets.

    Given an ImageFile (e.g. from an ImageField) and the label of a preset,
    return the ImageFile that represents the thumbnail with the settings
    specified in the preset.'''

    if preset_label == 'original':
        return file_
    args = get_preset_args(preset_label, presets, defaults)
    dimensions = args.pop('dimensions')
    return get_thumbnail(file_, dimensions, **args)
